#!/usr/bin/env python

"""
XSL-based form templating.
"""

import libxslt

"""
import libxml2

def quiet(context, s):
    pass

libxml2.registerErrorHandler(quiet, None)
"""

def path_to_node(node, name):

    """
    Generate an XSLForms path to the given 'node', using the given 'name' to
    complete the path if an attribute reference is required (otherwise 'name'
    will be None).
    """

    l = []
    # Skip attribute reference.
    if node.type == "attribute":
        node = node.parent
    # Manually insert the attribute name if defined.
    if name is not None:
        l.insert(0, name)
        l.insert(0, "/")
    # Element references.
    while node is not None and node.type != "document_xml":
        l.insert(0, str(int(node.xpathEval("count(preceding-sibling::*) + 1"))))
        l.insert(0, "#")
        l.insert(0, node.name)
        l.insert(0, "/")
        node = node.parent
    return "".join(l)

def this_position(context):

    """
    As a libxslt extension function, return a string containing the XSLForms
    path to the 'context' node, using the special "this-name" variable to
    complete the path if an attribute reference is required.
    """

    pctxt = libxslt.xpathParserContext(_obj=context)
    context = pctxt.context()
    node = context.contextNode()
    transform_context = context.transformContext()
    name_var = transform_context.variableLookup("this-name", None)
    if name_var is not None:
        name = name_var[0].content
    else:
        name = None
    return path_to_node(node, name)

libxslt.registerExtModuleFunction("this-position", "http://www.boddie.org.uk/ns/xmltools/template", this_position)

# vim: tabstop=4 expandtab shiftwidth=4
